#include "atca_hal.h"
#include "tx_api.h"

/** Standard Configuration Structure for ATECC608 devices */
const uint8_t atecc608_config[] = {
    0x01, 0x23, 0x00, 0x00, 0x00, 0x00, 0x60, 0x01, 0x00, 0x00, 0x00, 0x00, 0xEE, 0x01, 0x01, 0x00,
    0xC0, 0x00, 0x00, 0x01, 0x8F, 0x20, 0xC4, 0x44, 0x87, 0x20, 0x87, 0x20, 0x8F, 0x0F, 0xC4, 0x36,
    0x9F, 0x0F, 0x82, 0x20, 0x0F, 0x0F, 0xC4, 0x44, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F,
    0x0F, 0x0F, 0x0F, 0x0F, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x33, 0x00, 0x1C, 0x00, 0x13, 0x00, 0x13, 0x00, 0x7C, 0x00, 0x1C, 0x00, 0x3C, 0x00, 0x33, 0x00,
    0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x30, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x30, 0x00,
};

ATCA_STATUS hal_create_mutex(void ** ppMutex, char* pName)
{
    if (!ppMutex)
    {
        return ATCA_BAD_PARAM;
    }

    if ((*ppMutex) == NULL)
    {
        *ppMutex = malloc(sizeof(TX_MUTEX));
        if ((*ppMutex) == NULL)
        {
            return ATCA_FUNC_FAIL;
        }
    }
    uint8_t status = tx_mutex_create(*ppMutex, pName, TX_NO_INHERIT);
    if (status)
    {
        free(*ppMutex);
        return ATCA_FUNC_FAIL;
    }
    else
    {
        return ATCA_SUCCESS;
    }
}

ATCA_STATUS hal_destroy_mutex(void * pMutex)
{
    if (!pMutex)
    {
        return ATCA_BAD_PARAM;
    }
    tx_mutex_delete(pMutex);
    free(pMutex);
    return ATCA_SUCCESS;
}

ATCA_STATUS hal_lock_mutex(void * pMutex)
{
    if (!pMutex)
    {
        return ATCA_BAD_PARAM;
    }
    
    uint8_t status = tx_mutex_get(pMutex, TX_WAIT_FOREVER);
    if (status)
    {
        return ATCA_FUNC_FAIL;
    }
    else
    {
        return ATCA_SUCCESS;
    }
}

ATCA_STATUS hal_unlock_mutex(void * pMutex)
{
    if (!pMutex)
    {
        return ATCA_BAD_PARAM;
    }
    
    uint8_t status = tx_mutex_put(pMutex);
    if (status)
    {
        return ATCA_FUNC_FAIL;
    }
    else
    {
        return ATCA_SUCCESS;
    }
}
